(*	*	*	*	*	*	*	*	*	*	*	*	*	*	*	*	*
 *																*
 *					SU(3) color multiplets						*
 *						-	-	-	-	-						*
 *		Yang Li, <leeyoung@iastate.edu>, Aug. 30, 2012			*
 *																*
 *	This piece of Mathematica code compute the Young tableaux	*
 *	from direct product of fundamental representations and adj-	*
 *	oint representation. In QCD language, the code take the     *
 *  number of quarks, antiquarks and gluons as input. Then it   *
 !  produces the number of color color sinnglets.	            *
 *	The primary purpose of this code is to count the number of 	*
 *	colorsinglet states for a certain number of partons. 		*
 *	- - -														*
 *	to use the code, 											*
 *	SetDirectory[" where you store the code, must ends with /"]	*
 *	<<"colorsinglet.m"										*
 *	(* you shall see welcome information *)						*
 *	NumberOfSinglet[4, 2, 1]									*
 *	(*  compute number of singlet from 4 quarks, 2 antiquarks	*
 *		and 1 gluon *)											*
 *																*
 *	*	*	*	*	*	*	*	*	*	*	*	*	*	*	*	*)

BeginPackage["colorsinglet`"]

	(* * *
	 *information of public functions
	 * * *)

	Print["\t SU(3) Colorsinglet \n Yang Li <leeyoung@iastate.edu>, 08/31/2012 \n
		   \t version 5.0, updated 02/20/2013 "];
	
	NumberOfSinglet::usage =
	"NumberOfSinglet[quark, antiquark, gluon] counts the number of SU(3) colorsinglets
	from given number of quarks, antiquarks and gluons."

	GatherTableaux::usage = 
	" gather the same tableaux to a list"

	getDimension::usage = 
	"getDimension[tab] gives the dimension of a SU(3) Young tableau (irrep)." 
	
	CircleTimesQuark::usage =
	" CircleTimesQuark[irreps] computes irreps \[CircleTimes] (1,0), where (1,0) stands for funda-
	mental irrep quark, irreps is a list of irreps, in the multiplet notation form. 
	multiplet (n,m) is the same as Dykin notation {n+m,m,0}. The argument irreps 
	is assumed to be a list of irreps.
	example: { {{1,2}, 3}, {{1,1}, 2}, {{0,1},1} } represents irreps 3x(1,2) 2x(1,1) and a (0,1)"

	CircleTimesAntiquark::usage =
	" CircleTimesAquark[irreps] computes irreps \[CircleTimes] (0,1), where (0,1) stands for
	fundamental irrep antiquark. irreps is a list of Young tableaux in multiplet notation. 
	example: { {{1,2}, 3}, {{1,1}, 2}, {{0,1},1} } represents irreps 3x(1,2) 2x(1,1) and a (0,1)"

	CircleTimesGluon::usage = 
	"CircleTimesGluon[irreps] computes the list of irreps \[CircleTimes](1,1), where (1,1)
	stands for adjoint irrep gluon. 
	example: { {{1,2}, 3}, {{1,1}, 2}, {{0,1},1} } represents irreps 3x(1,2) 2x(1,1) and a (0,1)"


Begin["colorsinglet`Private`"]
	
	admissibleQ[ls_List, n_Integer]:=
		Module[
		{occ},
	(* *
	admissibleQ inspects list of letters ls and yields if ls admissible.
	A sequence of letters is admissible iff from beginning to any position, 
	for any two letters, the occurrence of the smaller one is always larger
	or equal to the occurrence of the larger one.
	
	example: {1, 1, 2, 1, 3, 2, 3} is admissible, {1, 1, 2, 3, 2, 2} is not.
	ls is a list of letters. Here, we use 1, 2, 3, ... to represent letters
	that can compare with each other. 
	n is an integer, the number of types of letters. 
	The letters in sequence ls, should be in [1, 2, ... n]
	* *)
	
		If[ n == 1, Return[True]]; (* trivial *)

		occ = Table[0, {n}]; (* initialize an occurrence list *)
		Return[
			Catch[
				Do[
					occ[[ letter ]]++; 
					If[ !OrderedQ[occ, #1 >= #2 &],
						Throw[False]
						] (* check admissiblity (= reversely-orderedness of occ) *)
					,{letter, ls}
					];
				True]
				];
		]
	
	getDimension[tableau_List] := Module[{n = tableau[[1]], m = tableau[[2]]}, (n+1)*(m+1)*(n+m+2)/2 ]

	GatherTableaux[tableaux_List] := Module[
		{ list },
	(* gather tableaux *)	
	(* example: tableaux={ {{2, 0}, 4}, {{2, 0},6}, {{0, 0}, 1} } *)
		(* list={ { {{2, 0}, 4}, {{2, 0}, 6} }, { {{0, 0}, 1} } } *)
		(* ls = { {{2, 0}, 4}, {{2, 0}, 6} }; 
			ls[[1,1]] = {2,0} is a multiplet representation of a Young 
			tableau;
		*)
		list=Gather[ tableaux, #1[[1]] == #2[[1]] &];
		Table[ 
			{ls[[1, 1]], Sum[ v[[2]], {v, ls}]}  
		,{ls, list}
		]
	]
	CircleTimesQuark[tableaux_List] := 
	(* direct product of a list of tableaux with quark irrep *)
	If[ Length@tableaux == 0,
		{ {{1,0}, 1} },
		GatherTableaux[Part[  
			Reap[
				Do[	
					Sow[tab+{{1,0},0}]; (* insert to row 1  *)	

					If[tab[[1,1]] > 0,
						Sow[tab+{{-1,1},0}] (* insert to row 2 *)
					];
					
					If[tab[[1,2]] > 0,
						Sow[tab+{{0,-1},0}] (* insert to the third row,
												cancel out one column*)
					];
				,{tab, tableaux}
				]
			]
		, 2, 1]
		]
	]

	CircleTimesAntiquark[tableaux_List] :=
	(* direct product of a list of tableaux with antiquark irrep *)
	If[ Length@tableaux == 0,
		{ {{0,1},1} },
		GatherTableaux[Part[
			Reap[
				Do[
					Sow[tab+{{0,1},0}]; (* a, b -> row 1, 2 *) 

					If[tab[[1,2]] > 0, (*a, b -> row 1, 3 *)
						Sow[tab+{{1,-1},0}]
					];
					
					If[tab[[1,1]] > 0,
						Sow[tab+{{-1,0},0}]; (* a, b -> row 2, 3 *)
						
					];
				,{tab, tableaux}
				]
			]
		, 2, 1]
		]
	]

	CircleTimesGluon[tableaux_List] := 
	(* direct product of a list of tableaux with gluon irrep *)
	If[ Length@tableaux == 0,
		{{{1,1},1}},
		GatherTableaux[
		Part[
			Reap[
				Do[
					Sow[tab+{{1,1},0}]; (* a, a, b --> 1, 1, 2 *)

					If[tab[[1,2]] > 1, 
						Sow[tab+{{1,-2},0}] (*a, a, b --> 1, 3, 3 *)
					];

					If[tab[[1,2]] > 0, 						
						Sow[tab+{{2,-1},0}];(* a, a, b --> 1, 1, 3 *)
						Sow[tab+{{0,0},0}]; (* a, a, b --> 1, 3, 2 *)
						If[ tab[[1,1]] > 0, 
							Sow[tab+{{-1,-1},0}] (* a, a, b --> 2, 3, 3 *)
						];
					];

					If[tab[[1,1]] > 0,
						Sow[tab+{{0,0},0}]; (* a, a, b --> 1, 2, 3 *)
						Sow[tab+{{-1,2},0}]; (* a, a, b --> 1, 2, 2 *)
						If[tab[[1,1]] > 1, (* a, a, b --> 2, 2, 3 *)
							Sow[tab+{{-2,1},0}]
						]
					];
				,{tab, tableaux}
				]
			]
		, 2, 1]
		]
	]


	SingletQ[tableau_List] := tableau[[1]] == 0
	
	CountSinglet[list_List] := If[ list[[1,1]] == 0, list[[2]], 0]

	PossibleQ[tableau_List, n_] := (2*tableau[[1]] + tableau[[2]] <= n);
		(* *
		PossibleQ[] makes a fast judgement if a tableau can make singlets with the 
		rest irreps. The rest irreps has n boxes. 
		* *)

	NumberOfSinglet[q_Integer, aq_Integer, g_Integer] := 
	Module[
		{pt = q + aq + g, quark = {1,0}, antiquark = {0,1}, gluon = {1,1}, 
		cst, tableaux, tab, CG, num},
		(* * 
		NumberOfSinglet[] count the number of SU(3) colorsinglets from q quarks, aq 
		antiquarks, g gluons.
		In this version, multiplet notation is used. When generating irreps, function
		PossibleQ[] is used to check if the current irrep may form a singlet with 
		all rest partons. 
		* *)

		num = q + 2*aq + 3*g; (* total number of squares *)
		(* Print["total boxes: ", num];	*)

		If[ pt > 0,
			
			tableaux = {{{0,0},1}};	(* initialization *)
			Do[
				(* tableaux = CircleTimesQuark[tableaux]; *)
				num -= 1; 
				tableaux = Select[CircleTimesQuark[tableaux], PossibleQ[#[[1]], num]& ]; 
				(* Print[tableaux]; *)  
				,{q}
			];

			Do[
				num -= 2;
				tableaux = Select[CircleTimesAntiquark[tableaux], PossibleQ[#[[1]], num]& ];
				(* Print[tableaux]; *) 
				,{aq}
			];

			Do[

				num -= 3;
				tableaux = Select[CircleTimesGluon[tableaux], PossibleQ[#[[1]], num]& ];
				(* Print[tableaux]; *) 
				,{g}
			];

	   	    (* Print["tableaux: ", tableaux]; *)
	   	    Return[Total[CountSinglet/@tableaux]]; (* pick colorsinglets *)
			,
			Return[0];
		];

	]


	CGSeries[q_, aq_, g_, mode_:0] := 
	Module[
		{pt = q + aq + g, quark = {1,0}, antiquark = {0,1}, gluon = {1,1}, 
		multiplet="", dimension="", tableaux, tab, CG, num},
		(* * 
		CGSeries[q, aq, g] gives the CS series of SU(3) irreps from product
		state of q quarks, aq antiquarks, g gluons.
		
		The function print latex source format string of the series in multiplet
		notation if mode = 1, in dimension notation if mode = 2, print both if
		mode = 3, print none if mode = 0.

		The function returns a list of irreps in multiplet form. 
		example: { {{0,1},3}, {{1,0},5}, {{1,1},4} } means there are 3 (0,1), 
		5 (1,0), 4 (1,1).
		* *)

		If[ pt > 0,
			
			tableaux = {{{0,0},1}};	(* initialization *)
			Do[
				tableaux = CircleTimesQuark[tableaux]; 
				(* Print[tableaux]; *)  
				,{q}
			];

			Do[
				tableaux = CircleTimesAntiquark[tableaux];
				(* Print[tableaux]; *) 
				,{aq}
			];

			Do[
				tableaux = CircleTimesGluon[tableaux];
				(* Print[tableaux]; *) 
				,{g}
			];

		];
	
	(* TODO mulitplet and dimension CS series strings *)	
		

		If[
			mode == 0, 
			Print[""];,
			If[
				mode == 1,
				Print[multiplet];,
				If[
				 	mode == 2,
				 	Print[dimension];,
				 	If[
				 		mode == 3,
						Print[multiplet];
						Print[dimension];
					 ]
				]
			]	
		]
	   	(* Print["tableaux: ", tableaux]; *)
		Return[tableaux];
	]
End[]

EndPackage[]


